<?php

namespace Drupal\chia\Services;

use Drupal\chia\Entity\Wallet;
use Drupal\chia\Exceptions\ChiaException;
use Drupal\chia\WalletRpcMethods;
use Drupal\Core\StreamWrapper\StreamWrapperManagerInterface;
use Drupal\file\Entity\File;
use GuzzleHttp\Client;
use GuzzleHttp\Exception\GuzzleException;
use GuzzleHttp\RequestOptions;

/**
 * Service description.
 */
class WalletRpc
{

  /**
   * @var StreamWrapperManagerInterface
   */
  protected StreamWrapperManagerInterface $streamWrapperManager;

  /**
   * @var Client[]
   *   The wallet client
   */
  protected array $walletClient = [];

  /**
   * Constructs a Rpc object.
   */
  public function __construct(StreamWrapperManagerInterface $streamWrapperManager)
  {
    $this->streamWrapperManager = $streamWrapperManager;
  }

  /**
   * Sends agnostic request to RPC server.
   *
   * @param \Drupal\chia\Entity\Wallet $wallet
   * @param string $method
   * @param array $params
   * @param string $requestMethod
   *
   * @return mixed
   * @throws \Drupal\chia\Exceptions\ChiaException
   * @throws \GuzzleHttp\Exception\GuzzleException
   */
  public function request(Wallet $wallet, string $method, array $params, string $requestMethod = 'POST'):mixed {
    $client = $this->newWalletClient($wallet);
    $response = $client->request($requestMethod, '/'.$method, [
      RequestOptions::BODY => json_encode((object) $params),
    ]);
    return json_decode($response->getBody()->getContents());
  }

  /**
   * @param Wallet|null $wallet
   * @return Client
   * @throws ChiaException
   */
  public function newWalletClient(Wallet $wallet = null): Client
  {
    if (!isset($this->walletClient[$wallet->id()])) {
      if (!isset($certPath) || !isset($keyPath)) {
        $walletCert = File::load($wallet->getWalletCert());
        $walletKey = File::load($wallet->getWalletKey());
        if ($walletCert && $walletKey) {
          $certPath = $this->streamWrapperManager->getViaUri($walletCert->getFileUri())->realpath();
          $keyPath = $this->streamWrapperManager->getViaUri($walletKey->getFileUri())->realpath();
        } else {
          throw new ChiaException("Unable to get wallet client : key or cert do not exist.");
        }
      }
      $options = [
        'base_uri' => $wallet->getWalletRpcAddress(),
        RequestOptions::HEADERS => [
          'Content-Type' => 'application/json',
        ],
        RequestOptions::VERIFY => false,
        RequestOptions::CONNECT_TIMEOUT => 1000,
        RequestOptions::CERT => $certPath,
        RequestOptions::SSL_KEY => $keyPath,
      ];
      $this->walletClient[$wallet->id()] = new Client($options);
    }
    return $this->walletClient[$wallet->id()];
  }


  /**
   * @throws GuzzleException
   * @throws ChiaException
   */
  public function getWallets(Wallet $wallet)
  {
    return $this->request($wallet, 'get_wallets',[
      'include_data' => true,
      'type' => 0,
    ]);
  }

  /**
   * @param \Drupal\chia\Entity\Wallet $wallet
   *
   * @return array
   */
  public function getSyncStatus(Wallet $wallet) {
    return $this->request($wallet, 'get_sync_status',[]);
  }

  /**
   * @param Wallet $wallet
   * @param int $walletId
   * @return mixed
   * @throws ChiaException
   * @throws GuzzleException
   */
  public function getNextAddress(Wallet $wallet, $walletId = 1): string
  {
    $result = $this->request($wallet, WalletRpcMethods::GET_NEXT_ADDRESS, [
      'wallet_id' => (int) $walletId,
      'new_address' => true,
    ]);
    // If failed, then get last used address
    if (!$result->success) {
      $result = $this->request($wallet, WalletRpcMethods::GET_NEXT_ADDRESS, [
        'wallet_id' => (string) $walletId,
        'new_address' => false,
      ]);
    }
    return $result->address;
  }

  /**
   * @throws GuzzleException|ChiaException
   */
  public function getLastTransactions(Wallet $wallet, $walletId = 1, $toAddress = null)
  {
    $params = [
      'start' => 0,
      'end' => 50,
      'reverse' => true,
      'wallet_id' => $walletId,
    ];
    if (isset($toAddress)) {
      $params['to_address'] = $toAddress;
    }
    return $this->request($wallet, WalletRpcMethods::GET_TRANSACTIONS, $params);
  }

  /**
   * @param $ids
   *
   * @return array
   */
  public function getWalletStates($ids = NULL): array {
    $wallets = Wallet::loadMultiple($ids);
    $newWalletState = [];
    foreach ($wallets as $wallet) {
      try {
        $status = $this->getSyncStatus($wallet);
        $newWalletState[$wallet->id()] = $status->success ? 'ok' : 'warning';
      } catch(\Exception $exc) {
        $newWalletState[$wallet->id()] = 'error';
      }
    }
    return $newWalletState;
  }

}
